@@ -10,13 +10,10 @@ ax.add_feature().
 
 """
 
-from __future__ import (absolute_import, division, print_function)
-
 from abc import ABCMeta, abstractmethod
 
 import numpy as np
 import shapely.geometry as sgeom
-import six
 
 import cartopy.io.shapereader as shapereader
 import cartopy.crs
@@ -33,7 +30,6 @@ feature being plotted.
 
 """
 
-
 _NATURAL_EARTH_GEOM_CACHE = {}
 """
 Caches a mapping between (name, category, scale) and a tuple of the
@@ -46,7 +42,7 @@ same projection.
 """
 
 
-class Feature(six.with_metaclass(ABCMeta)):
+class Feature(metaclass=ABCMeta):
     """
     Represents a collection of points, lines and polygons with convenience
     methods for common drawing and filtering operations.
@@ -113,7 +109,7 @@ class Feature(six.with_metaclass(ABCMeta)):
             return self.geometries()
 
 
-class Scaler(object):
+class Scaler:
     """
     General object for handling the scale of the geometries used in a Feature.
     """
@@ -169,7 +165,7 @@ class AdaptiveScaler(Scaler):
         'fine'
 
         """
-        super(AdaptiveScaler, self).__init__(default_scale)
+        super().__init__(default_scale)
         self._default_scale = default_scale
         # Upper limit on extent in degrees.
         self._limits = limits
@@ -217,7 +213,7 @@ class ShapelyFeature(Feature):
             Keyword arguments to be used when drawing this feature.
 
         """
-        super(ShapelyFeature, self).__init__(crs, **kwargs)
+        super().__init__(crs, **kwargs)
         self._geoms = tuple(geometries)
 
     def geometries(self):
@@ -250,13 +246,12 @@ class NaturalEarthFeature(Feature):
             Keyword arguments to be used when drawing this feature.
 
         """
-        super(NaturalEarthFeature, self).__init__(cartopy.crs.PlateCarree(),
-                                                  **kwargs)
+        super().__init__(cartopy.crs.PlateCarree(), **kwargs)
         self.category = category
         self.name = name
 
         # Cast the given scale to a (constant) Scaler if a string is passed.
-        if isinstance(scale, six.string_types):
+        if isinstance(scale, str):
             scale = Scaler(scale)
 
         self.scaler = scale
@@ -270,7 +265,7 @@ class NaturalEarthFeature(Feature):
     def _validate_scale(self):
         if self.scale not in ('110m', '50m', '10m'):
             raise ValueError(
-                '{} is not a valid Natural Earth scale. '.format(self.scale) +
+                f'{self.scale!r} is not a valid Natural Earth scale. '
                 'Valid scales are "110m", "50m", and "10m".'
             )
 
@@ -299,7 +294,7 @@ class NaturalEarthFeature(Feature):
         If extent is None, the method returns all geometries for this dataset.
         """
         self.scaler.scale_from_extent(extent)
-        return super(NaturalEarthFeature, self).intersecting_geometries(extent)
+        return super().intersecting_geometries(extent)
 
     def with_scale(self, new_scale):
         """
@@ -329,7 +324,7 @@ class GSHHSFeature(Feature):
         The dataset scale. One of 'auto', 'coarse', 'low', 'intermediate',
         'high, or 'full' (default is 'auto').
     levels
-        A list of integers 1-4 corresponding to the desired GSHHS feature
+        A list of integers 1-6 corresponding to the desired GSHHS feature
         levels to draw (default is [1] which corresponds to coastlines).
 
     Other Parameters
@@ -351,11 +346,11 @@ class GSHHSFeature(Feature):
 
     """
     def __init__(self, scale='auto', levels=None, **kwargs):
-        super(GSHHSFeature, self).__init__(cartopy.crs.PlateCarree(), **kwargs)
+        super().__init__(cartopy.crs.PlateCarree(), **kwargs)
 
         if scale not in ('auto', 'a', 'coarse', 'c', 'low', 'l',
                          'intermediate', 'i', 'high', 'h', 'full', 'f'):
-            raise ValueError("Unknown GSHHS scale '{}'.".format(scale))
+            raise ValueError(f"Unknown GSHHS scale {scale!r}.")
         self._scale = scale
 
         if levels is None:
@@ -363,8 +358,7 @@ class GSHHSFeature(Feature):
         self._levels = set(levels)
         unknown_levels = self._levels.difference([1, 2, 3, 4])
         if unknown_levels:
-            raise ValueError("Unknown GSHHS levels "
-                             "'{}'.".format(unknown_levels))
+            raise ValueError(f"Unknown GSHHS levels {unknown_levels!r}.")
 
         # Default kwargs
         self._kwargs.setdefault('edgecolor', 'black')
@@ -450,13 +444,13 @@ class WFSFeature(Feature):
         try:
             from cartopy.io.ogc_clients import WFSGeometrySource
         except ImportError as e:
-            six.raise_from(ImportError(
+            raise ImportError(
                 'WFSFeature requires additional dependencies. If installed '
-                'via pip, try `pip install cartopy[ows]`.\n'), e)
+                'via pip, try `pip install cartopy[ows]`.\n') from e
 
         self.source = WFSGeometrySource(wfs, features)
         crs = self.source.default_projection()
-        super(WFSFeature, self).__init__(crs, **kwargs)
+        super().__init__(crs, **kwargs)
         # Default kwargs
         self._kwargs.setdefault('edgecolor', 'black')
         self._kwargs.setdefault('facecolor', 'none')
