@@ -1,21 +1,9 @@
-# (C) British Crown Copyright 2011 - 2019, Met Office
+# Copyright Cartopy Contributors
 #
-# This file is part of cartopy.
-#
-# cartopy is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# cartopy is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with cartopy.  If not, see <https://www.gnu.org/licenses/>.
+# This file is part of Cartopy and is released under the LGPL license.
+# See COPYING and COPYING.LESSER in the root of the repository for full
+# licensing details.
 
-from __future__ import (absolute_import, division, print_function)
 
 import collections
 import glob
@@ -24,7 +12,6 @@ import os.path
 import numpy as np
 from PIL import Image
 import shapely.geometry as sgeom
-from six.moves import zip
 
 
 _img_class_attrs = ['filename', 'extent', 'origin', 'pixel_size']
@@ -43,7 +30,7 @@ class Img(collections.namedtuple('Img', _img_class_attrs)):
             if isinstance(item, list):
                 item = tuple(item)
             new_kwargs[k] = item
-        return super(Img, cls).__new__(cls, *new_args, **new_kwargs)
+        return super().__new__(cls, *new_args, **new_kwargs)
 
     def __init__(self, *args, **kwargs):
         """
@@ -129,17 +116,15 @@ class Img(collections.namedtuple('Img', _img_class_attrs)):
         froot, fext = os.path.splitext(fname)
         # If there was no extension to the filename.
         if froot == fname:
-            result = ['{}.{}'.format(fname, 'w'),
-                      '{}.{}'.format(fname, 'W')]
+            result = [f'{fname}.w', f'{fname}.W']
         else:
             fext = fext[1::].lower()
             if len(fext) < 3:
-                result = ['{}.{}'.format(fname, 'w'),
-                          '{}.{}'.format(fname, 'W')]
+                result = [f'{fname}.w', f'{fname}.W']
             else:
-                fext_types = [fext + 'w', fext[0] + fext[-1] + 'w']
+                fext_types = [f'{fext}w', f'{fext[0]}{fext[-1]}w']
                 fext_types.extend([ext.upper() for ext in fext_types])
-                result = ['{}.{}'.format(froot, ext) for ext in fext_types]
+                result = [f'{froot}.{ext}' for ext in fext_types]
 
         def _convert_basename(name):
             dirname, basename = os.path.split(name)
@@ -200,7 +185,7 @@ class Img(collections.namedtuple('Img', _img_class_attrs)):
         return (min_x, max_x, min_y, max_y), pix_size
 
 
-class ImageCollection(object):
+class ImageCollection:
     def __init__(self, name, crs, images=None):
         """
         Represents a collection of images at the same logical level.
@@ -253,13 +238,13 @@ class ImageCollection(object):
                 if os.path.exists(fworld):
                     break
             else:
-                msg = 'Image file {!r} has no associated world file'
-                raise ValueError(msg.format(img))
+                raise ValueError(
+                    f'Image file {img!r} has no associated world file')
 
             self.images.append(img_class.from_world_file(img, fworld))
 
 
-class NestedImageCollection(object):
+class NestedImageCollection:
     def __init__(self, name, crs, collections, _ancestry=None):
         """
         Represents a complex nest of ImageCollections.
@@ -286,7 +271,7 @@ class NestedImageCollection(object):
 
         """
         # NOTE: all collections must have the same crs.
-        _names = set([collection.name for collection in collections])
+        _names = {collection.name for collection in collections}
         assert len(_names) == len(collections), \
             'The collections must have unique names.'
 
@@ -367,14 +352,14 @@ class NestedImageCollection(object):
         # XXX Copied from cartopy.io.img_tiles
         if target_z not in self._collections_by_name:
             # TODO: Handle integer depths also?
-            msg = '{!r} is not one of the possible collections.'
-            raise ValueError(msg.format(target_z))
+            raise ValueError(
+                f'{target_z!r} is not one of the possible collections.')
 
         tiles = []
         for tile in self.find_images(target_domain, target_z):
             try:
                 img, extent, origin = self.get_image(tile)
-            except IOError:
+            except OSError:
                 continue
 
             img = np.array(img)
@@ -421,8 +406,8 @@ class NestedImageCollection(object):
         # XXX Copied from cartopy.io.img_tiles
         if target_z not in self._collections_by_name:
             # TODO: Handle integer depths also?
-            msg = '{!r} is not one of the possible collections.'
-            raise ValueError(msg.format(target_z))
+            raise ValueError(
+                f'{target_z!r} is not one of the possible collections.')
 
         if start_tiles is None:
             start_tiles = ((self._collections[0].name, img)
@@ -437,10 +422,9 @@ class NestedImageCollection(object):
                     yield start_tile
                 else:
                     for tile in self.subtiles(start_tile):
-                        for result in self.find_images(target_domain,
-                                                       target_z,
-                                                       start_tiles=[tile]):
-                            yield result
+                        yield from self.find_images(target_domain,
+                                                    target_z,
+                                                    start_tiles=[tile])
 
     def subtiles(self, collection_image):
         """
