@@ -1,19 +1,8 @@
-# (C) British Crown Copyright 2014 - 2020, Met Office
+# Copyright Cartopy Contributors
 #
-# This file is part of cartopy.
-#
-# cartopy is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# cartopy is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with cartopy.  If not, see <https://www.gnu.org/licenses/>.
+# This file is part of Cartopy and is released under the LGPL license.
+# See COPYING and COPYING.LESSER in the root of the repository for full
+# licensing details.
 """
 Implements RasterSource classes which can retrieve imagery from web services
 such as WMS and WMTS.
@@ -23,15 +12,10 @@ The matplotlib interface can make use of RasterSources via the
 with additional specific methods which make use of this for WMS and WMTS
 (:meth:`~cartopy.mpl.geoaxes.GeoAxes.add_wms` and
 :meth:`~cartopy.mpl.geoaxes.GeoAxes.add_wmts`). An example of using WMTS in
-this way can be found at :ref:`sphx_glr_gallery_wmts.py`.
-
+this way can be found at :ref:`sphx_glr_gallery_web_services_wmts.py`.
 
 """
 
-from __future__ import (absolute_import, division, print_function)
-
-import six
-
 import collections
 import io
 import math
@@ -178,7 +162,7 @@ def _target_extents(extent, requested_projection, available_projection):
 
     # Return the polygons' rectangular bounds as extent tuples.
     target_extents = []
-    for poly in polys:
+    for poly in polys.geoms:
         min_x, min_y, max_x, max_y = poly.bounds
         if fudge_mode:
             # If we shrunk the request area before, then here we
@@ -235,10 +219,10 @@ class WMSRasterSource(RasterSource):
         if WebMapService is None:
             raise ImportError(_OWSLIB_REQUIRED)
 
-        if isinstance(service, six.string_types):
+        if isinstance(service, str):
             service = WebMapService(service)
 
-        if isinstance(layers, six.string_types):
+        if isinstance(layers, str):
             layers = [layers]
 
         if getmap_extra_kwargs is None:
@@ -248,8 +232,8 @@ class WMSRasterSource(RasterSource):
             raise ValueError('One or more layers must be defined.')
         for layer in layers:
             if layer not in service.contents:
-                raise ValueError('The {!r} layer does not exist in '
-                                 'this service.'.format(layer))
+                raise ValueError(f'The {layer!r} layer does not exist in '
+                                 'this service.')
 
         #: The OWSLib WebMapService instance.
         self.service = service
@@ -273,7 +257,7 @@ class WMSRasterSource(RasterSource):
 
         """
         contents = self.service.contents
-        for proj, srs in six.iteritems(_CRS_TO_OGC_SRS):
+        for proj, srs in _CRS_TO_OGC_SRS.items():
             missing = any(srs not in contents[layer].crsOptions for
                           layer in self.layers)
             if not missing:
@@ -303,7 +287,7 @@ class WMSRasterSource(RasterSource):
                                      target_resolution)
 
     def fetch_raster(self, projection, extent, target_resolution):
-        target_resolution = [int(np.ceil(val)) for val in target_resolution]
+        target_resolution = [math.ceil(val) for val in target_resolution]
         wms_srs = self._native_srs(projection)
         if wms_srs is not None:
             wms_proj = projection
@@ -374,8 +358,8 @@ class WMTSRasterSource(RasterSource):
         try:
             layer = wmts.contents[layer_name]
         except KeyError:
-            raise ValueError('Invalid layer name {!r} for WMTS at {!r}'.format(
-                layer_name, wmts.url))
+            raise ValueError(
+                f'Invalid layer name {layer_name!r} for WMTS at {wmts.url!r}')
 
         #: The OWSLib WebMapTileService instance.
         self.wmts = wmts
@@ -422,11 +406,11 @@ class WMTSRasterSource(RasterSource):
                         break
                 if matrix_set_name is None:
                     # Fail completely.
-                    available_urns = sorted(set(
+                    available_urns = sorted({
                         self.wmts.tilematrixsets[name].crs
-                        for name in matrix_set_names))
+                        for name in matrix_set_names})
                     msg = 'Unable to find tile matrix for projection.'
-                    msg += '\n    Projection: ' + str(target_projection)
+                    msg += f'\n    Projection: {target_projection}'
                     msg += '\n    Available tile CRS URNs:'
                     msg += '\n        ' + '\n        '.join(available_urns)
                     raise ValueError(msg)
@@ -637,7 +621,7 @@ class WMTSRasterSource(RasterSource):
         return big_img, img_extent
 
 
-class WFSGeometrySource(object):
+class WFSGeometrySource:
     """Web Feature Service (WFS) retrieval for Cartopy."""
 
     def __init__(self, service, features, getfeature_extra_kwargs=None):
@@ -658,10 +642,10 @@ class WFSGeometrySource(object):
         if WebFeatureService is None:
             raise ImportError(_OWSLIB_REQUIRED)
 
-        if isinstance(service, six.string_types):
+        if isinstance(service, str):
             service = WebFeatureService(service)
 
-        if isinstance(features, six.string_types):
+        if isinstance(features, str):
             features = [features]
 
         if getfeature_extra_kwargs is None:
@@ -671,8 +655,8 @@ class WFSGeometrySource(object):
             raise ValueError('One or more features must be specified.')
         for feature in features:
             if feature not in service.contents:
-                raise ValueError('The {!r} feature does not exist in this '
-                                 'service.'.format(feature))
+                raise ValueError(
+                    f'The {feature!r} feature does not exist in this service.')
 
         self.service = service
         self.features = features
@@ -688,21 +672,22 @@ class WFSGeometrySource(object):
         """
         # Using first element in crsOptions (default).
         if self._default_urn is None:
-            default_urn = set(self.service.contents[feature].crsOptions[0] for
-                              feature in self.features)
+            default_urn = {self.service.contents[feature].crsOptions[0] for
+                           feature in self.features}
             if len(default_urn) != 1:
                 ValueError('Failed to find a single common default SRS '
                            'across all features (typenames).')
             else:
                 default_urn = default_urn.pop()
 
-            if six.text_type(default_urn) not in _URN_TO_CRS:
-                raise ValueError('Unknown mapping from SRS/CRS_URN {!r} to '
-                                 'cartopy projection.'.format(default_urn))
+            if str(default_urn) not in _URN_TO_CRS:
+                raise ValueError(
+                    f'Unknown mapping from SRS/CRS_URN {default_urn!r} to '
+                    'cartopy projection.')
 
             self._default_urn = default_urn
 
-        return _URN_TO_CRS[six.text_type(self._default_urn)]
+        return _URN_TO_CRS[str(self._default_urn)]
 
     def fetch_geometries(self, projection, extent):
         """
@@ -727,7 +712,7 @@ class WFSGeometrySource(object):
         """
         if self.default_projection() != projection:
             raise ValueError('Geometries are only available in projection '
-                             '{!r}.'.format(self.default_projection()))
+                             f'{self.default_projection()!r}.')
 
         min_x, max_x, min_y, max_y = extent
         response = self.service.getfeature(typename=self.features,
@@ -748,15 +733,14 @@ class WFSGeometrySource(object):
                 if srs in _URN_TO_CRS:
                     geom_proj = _URN_TO_CRS[srs]
                     if geom_proj != projection:
-                        raise ValueError('The geometries are not in expected '
-                                         'projection. Expected {!r}, got '
-                                         '{!r}.'.format(projection, geom_proj))
+                        raise ValueError(
+                            'The geometries are not in expected projection. '
+                            f'Expected {projection!r}, got {geom_proj!r}.')
                 else:
-                    msg = 'Unable to verify matching projections due ' \
-                          'to incomplete mappings from SRS identifiers ' \
-                          'to cartopy projections. The geometries have ' \
-                          'an SRS of {!r}.'.format(srs)
-                    warnings.warn(msg)
+                    warnings.warn(
+                        'Unable to verify matching projections due to '
+                        'incomplete mappings from SRS identifiers to cartopy '
+                        f'projections. The geometries have an SRS of {srs!r}.')
         return geoms
 
     def _to_shapely_geoms(self, response):
@@ -781,21 +765,21 @@ class WFSGeometrySource(object):
         points_data = []
         tree = ElementTree.parse(response)
 
-        for node in tree.findall('.//{}msGeometry'.format(_MAP_SERVER_NS)):
+        for node in tree.findall(f'.//{_MAP_SERVER_NS}msGeometry'):
             # Find LinearRing geometries in our msGeometry node.
-            find_str = './/{gml}LinearRing'.format(gml=_GML_NS)
+            find_str = f'.//{_GML_NS}LinearRing'
             if self._node_has_child(node, find_str):
                 data = self._find_polygon_coords(node, find_str)
                 linear_rings_data.extend(data)
 
             # Find LineString geometries in our msGeometry node.
-            find_str = './/{gml}LineString'.format(gml=_GML_NS)
+            find_str = f'.//{_GML_NS}LineString'
             if self._node_has_child(node, find_str):
                 data = self._find_polygon_coords(node, find_str)
                 linestrings_data.extend(data)
 
             # Find Point geometries in our msGeometry node.
-            find_str = './/{gml}Point'.format(gml=_GML_NS)
+            find_str = f'.//{_GML_NS}Point'
             if self._node_has_child(node, find_str):
                 data = self._find_polygon_coords(node, find_str)
                 points_data.extend(data)
@@ -839,8 +823,8 @@ class WFSGeometrySource(object):
             x, y = [], []
 
             # We can have nodes called `coordinates` or `coord`.
-            coordinates_find_str = '{}coordinates'.format(_GML_NS)
-            coords_find_str = '{}coord'.format(_GML_NS)
+            coordinates_find_str = f'{_GML_NS}coordinates'
+            coords_find_str = f'{_GML_NS}coord'
 
             if self._node_has_child(polygon, coordinates_find_str):
                 points = polygon.findtext(coordinates_find_str)
@@ -851,8 +835,8 @@ class WFSGeometrySource(object):
                     y.append(float(y_val))
             elif self._node_has_child(polygon, coords_find_str):
                 for coord in polygon.findall(coords_find_str):
-                    x.append(float(coord.findtext('{}X'.format(_GML_NS))))
-                    y.append(float(coord.findtext('{}Y'.format(_GML_NS))))
+                    x.append(float(coord.findtext(f'{_GML_NS}X')))
+                    y.append(float(coord.findtext(f'{_GML_NS}Y')))
             else:
                 raise ValueError('Unable to find or parse coordinate values '
                                  'from the XML.')
