@@ -1,20 +1,8 @@
-# (C) British Crown Copyright 2011 - 2020, Met Office
+# Copyright Cartopy Contributors
 #
-# This file is part of cartopy.
-#
-# cartopy is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# cartopy is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with cartopy.  If not, see <https://www.gnu.org/licenses/>.
-
+# This file is part of Cartopy and is released under the LGPL license.
+# See COPYING and COPYING.LESSER in the root of the repository for full
+# licensing details.
 
 """
 Combine the shapefile access of pyshp with the
@@ -39,15 +27,14 @@ geometry representation of shapely:
 
 """
 
-from __future__ import (absolute_import, division, print_function)
-
 import glob
+import io
 import itertools
 import os
+from urllib.error import HTTPError
 
 import shapely.geometry as sgeom
 import shapefile
-import six
 
 from cartopy.io import Downloader
 from cartopy import config
@@ -61,7 +48,7 @@ except ImportError:
 __all__ = ['Reader', 'Record']
 
 
-class Record(object):
+class Record:
     """
     A single logical entry from a shapefile, combining the attributes with
     their associated geometry.
@@ -85,10 +72,10 @@ class Record(object):
         self._fields = fields
 
     def __repr__(self):
-        return '<Record: %r, %r, <fields>>' % (self.geometry, self.attributes)
+        return f'<Record: {self.geometry!r}, {self.attributes!r}, <fields>>'
 
     def __str__(self):
-        return 'Record(%s, %s, <fields>)' % (self.geometry, self.attributes)
+        return f'Record({self.geometry}, {self.attributes}, <fields>)'
 
     @property
     def bounds(self):
@@ -127,7 +114,7 @@ class FionaRecord(Record):
         self._bounds = geometry.bounds
 
 
-class BasicReader(object):
+class BasicReader:
     """
     Provide an interface for accessing the contents of a shapefile.
 
@@ -148,7 +135,7 @@ class BasicReader(object):
         return self._reader.close()
 
     def __len__(self):
-        return self._reader.numRecords
+        return len(self._reader)
 
     def geometries(self):
         """
@@ -162,8 +149,7 @@ class BasicReader(object):
         :meth:`~Record.geometry` method.
 
         """
-        for i in range(self._reader.numRecords):
-            shape = self._reader.shape(i)
+        for shape in self._reader.iterShapes():
             # Skip the shape that can not be represented as geometry.
             if shape.shapeType != shapefile.NULL:
                 yield sgeom.shape(shape)
@@ -175,14 +161,12 @@ class BasicReader(object):
         """
         # Ignore the "DeletionFlag" field which always comes first
         fields = self._reader.fields[1:]
-        field_names = [field[0] for field in fields]
-        for i in range(self._reader.numRecords):
-            shape_record = self._reader.shapeRecord(i)
-            attributes = dict(zip(field_names, shape_record.record))
+        for shape_record in self._reader.iterShapeRecords():
+            attributes = shape_record.record.as_dict()
             yield Record(shape_record.shape, attributes, fields)
 
 
-class FionaReader(object):
+class FionaReader:
     """
     Provides an interface for accessing the contents of a shapefile
     with the fiona library, which has a much faster reader than pyshp.
@@ -267,7 +251,7 @@ else:
 def natural_earth(resolution='110m', category='physical', name='coastline'):
     """
     Return the path to the requested natural earth shapefile,
-    downloading and unziping if necessary.
+    downloading and unzipping if necessary.
 
     To identify valid components for this function, either browse
     NaturalEarthData.com, or if you know what you are looking for, go to
@@ -307,11 +291,10 @@ class NEShpDownloader(Downloader):
     """
     FORMAT_KEYS = ('config', 'resolution', 'category', 'name')
 
-    # Define the NaturalEarth URL template. The natural earth website
-    # returns a 302 status if accessing directly, so we use the naciscdn
-    # URL directly.
-    _NE_URL_TEMPLATE = ('https://naciscdn.org/naturalearth/{resolution}'
-                        '/{category}/ne_{resolution}_{name}.zip')
+    # Define the NaturalEarth URL template. Shapefiles are hosted on AWS since
+    # 2021: https://github.com/nvkelso/natural-earth-vector/issues/445
+    _NE_URL_TEMPLATE = ('https://naturalearth.s3.amazonaws.com/'
+                        '{resolution}_{category}/ne_{resolution}_{name}.zip')
 
     def __init__(self,
                  url_template=_NE_URL_TEMPLATE,
@@ -329,7 +312,7 @@ class NEShpDownloader(Downloader):
         natural earth zip file.
 
         """
-        for ext in ['.shp', '.dbf', '.shx']:
+        for ext in ['.shp', '.dbf', '.shx', '.prj', '.cpg']:
             yield ('ne_{resolution}_{name}'
                    '{extension}'.format(extension=ext, **format_dict))
 
@@ -349,7 +332,7 @@ class NEShpDownloader(Downloader):
 
         shapefile_online = self._urlopen(url)
 
-        zfh = ZipFile(six.BytesIO(shapefile_online.read()), 'r')
+        zfh = ZipFile(io.BytesIO(shapefile_online.read()), 'r')
 
         for member_path in self.zip_file_contents(format_dict):
             ext = os.path.splitext(member_path)[1]
@@ -397,7 +380,7 @@ config['downloaders'].setdefault(_ne_key,
 def gshhs(scale='c', level=1):
     """
     Return the path to the requested GSHHS shapefile,
-    downloading and unziping if necessary.
+    downloading and unzipping if necessary.
 
     """
     # Get hold of the Downloader (typically a GSHHSShpDownloader instance)
@@ -421,17 +404,19 @@ class GSHHSShpDownloader(Downloader):
     """
     FORMAT_KEYS = ('config', 'scale', 'level')
 
-    _GSHHS_URL_TEMPLATE = ('https://www.ngdc.noaa.gov/mgg/shorelines/data/'
-                           'gshhs/oldversions/version2.2.0/'
-                           'GSHHS_shp_2.2.0.zip')
+    gshhs_version = '2.3.7'
+
+    _GSHHS_URL_TEMPLATE = (
+        'https://www.ngdc.noaa.gov/mgg/shorelines/data/'
+        f'gshhs/latest/gshhg-shp-{gshhs_version}.zip'
+    )
 
     def __init__(self,
                  url_template=_GSHHS_URL_TEMPLATE,
                  target_path_template=None,
                  pre_downloaded_path_template=''):
-        super(GSHHSShpDownloader, self).__init__(url_template,
-                                                 target_path_template,
-                                                 pre_downloaded_path_template)
+        super().__init__(url_template, target_path_template,
+                         pre_downloaded_path_template)
 
     def zip_file_contents(self, format_dict):
         """
@@ -449,15 +434,43 @@ class GSHHSShpDownloader(Downloader):
 
         # Download archive.
         url = self.url(format_dict)
-        shapefile_online = self._urlopen(url)
-        zfh = ZipFile(six.BytesIO(shapefile_online.read()), 'r')
+        try:
+            shapefile_online = self._urlopen(url)
+        # error handling:
+        except HTTPError:
+            try:
+                """
+                case if GSHHS has had an update
+                without changing the naming convention
+                """
+                url = (
+                    f'https://www.ngdc.noaa.gov/mgg/shorelines/data/'
+                    f'gshhs/oldversions/version{self.gshhs_version}/'
+                    f'gshhg-shp-{self.gshhs_version}.zip'
+                )
+                shapefile_online = self._urlopen(url)
+            except HTTPError:
+                """
+                case if GSHHS has had an update
+                with changing the naming convention
+                """
+                url = (
+                    'https://www.ngdc.noaa.gov/mgg/shorelines/data/'
+                    'gshhs/oldversions/version2.3.6/'
+                    'gshhg-shp-2.3.6.zip'
+                )
+                shapefile_online = self._urlopen(url)
+        zfh = ZipFile(io.BytesIO(shapefile_online.read()), 'r')
         shapefile_online.close()
 
         # Iterate through all scales and levels and extract relevant files.
         modified_format_dict = dict(format_dict)
         scales = ('c', 'l', 'i', 'h', 'f')
-        levels = (1, 2, 3, 4)
+        levels = (1, 2, 3, 4, 5, 6)
         for scale, level in itertools.product(scales, levels):
+            # the combination c4 does not occur for some reason
+            if scale == "c" and level == 4:
+                continue
             modified_format_dict.update({'scale': scale, 'level': level})
             target_path = self.target_path(modified_format_dict)
             target_dir = os.path.dirname(target_path)
@@ -496,7 +509,7 @@ class GSHHSShpDownloader(Downloader):
         self.acquire_all_resources(format_dict)
         if not os.path.exists(target_path):
             raise RuntimeError('Failed to download and extract GSHHS '
-                               'shapefile to {!r}.'.format(target_path))
+                               f'shapefile to {target_path!r}.')
         return target_path
 
     @staticmethod
