@@ -1,19 +1,8 @@
-# (C) British Crown Copyright 2011 - 2019, Met Office
+# Copyright Cartopy Contributors
 #
-# This file is part of cartopy.
-#
-# cartopy is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# cartopy is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with cartopy.  If not, see <https://www.gnu.org/licenses/>.
+# This file is part of Cartopy and is released under the LGPL license.
+# See COPYING and COPYING.LESSER in the root of the repository for full
+# licensing details.
 
 """
 The Shuttle Radar Topography Mission (SRTM) is an international research
@@ -25,13 +14,11 @@ database of Earth prior to the release of the ASTER GDEM in 2009.
 
 """
 
-from __future__ import (absolute_import, division, print_function)
-
+import io
 import os
 import warnings
 
 import numpy as np
-import six
 
 from cartopy import config
 import cartopy.crs as ccrs
@@ -70,7 +57,7 @@ class _SRTMSource(RasterSource):
             self._shape = (3601, 3601)
         else:
             raise ValueError(
-                'Resolution is an unexpected value ({}).'.format(resolution))
+                f'Resolution is an unexpected value ({resolution}).')
         self._resolution = resolution
 
         #: The CRS of the underlying SRTM data.
@@ -82,7 +69,7 @@ class _SRTMSource(RasterSource):
 
         if self.downloader is None:
             self.downloader = Downloader.from_config(
-                ('SRTM', 'SRTM' + str(resolution)))
+                ('SRTM', f'SRTM{resolution}'))
 
         #: A tuple of (max_x_tiles, max_y_tiles).
         self._max_tiles = (max_nx, max_ny)
@@ -96,9 +83,8 @@ class _SRTMSource(RasterSource):
 
         """
         if not self.validate_projection(projection):
-            raise ValueError(
-                'Unsupported projection for the SRTM{} source.'.format(
-                    self._resolution))
+            raise ValueError(f'Unsupported projection for the '
+                             f'SRTM{self._resolution} source.')
 
         min_x, max_x, min_y, max_y = extent
         min_x, min_y = np.floor([min_x, min_y])
@@ -107,15 +93,13 @@ class _SRTMSource(RasterSource):
         skip = False
         if nx > self._max_tiles[0]:
             warnings.warn(
-                'Required SRTM{} tile count ({}) exceeds maximum ({}). '
-                'Increase max_nx limit.'.format(self._resolution, nx,
-                                                self._max_tiles[0]))
+                f'Required SRTM{self._resolution} tile count ({nx}) exceeds '
+                f'maximum ({self._max_tiles[0]}). Increase max_nx limit.')
             skip = True
         if ny > self._max_tiles[1]:
             warnings.warn(
-                'Required SRTM{} tile count ({}) exceeds maximum ({}). '
-                'Increase max_ny limit.'.format(self._resolution, ny,
-                                                self._max_tiles[1]))
+                f'Required SRTM{self._resolution} tile count ({ny}) exceeds '
+                f'maximum ({self._max_tiles[1]}). Increase max_ny limit.')
             skip = True
         if skip:
             return []
@@ -137,7 +121,7 @@ class _SRTMSource(RasterSource):
         y = '%s%02d' % ('N' if lat >= 0 else 'S', abs(int(lat)))
 
         srtm_downloader = Downloader.from_config(
-            ('SRTM', 'SRTM' + str(self._resolution)))
+            ('SRTM', f'SRTM{self._resolution}'))
         params = {'config': config, 'resolution': self._resolution,
                   'x': x, 'y': y}
 
@@ -205,8 +189,8 @@ class SRTM3Source(_SRTMSource):
             producing a taller composite for this RasterSource.
 
         """
-        super(SRTM3Source, self).__init__(resolution=3, downloader=downloader,
-                                          max_nx=max_nx, max_ny=max_ny)
+        super().__init__(resolution=3, downloader=downloader,
+                         max_nx=max_nx, max_ny=max_ny)
 
 
 class SRTM1Source(_SRTMSource):
@@ -232,8 +216,8 @@ class SRTM1Source(_SRTMSource):
             producing a taller composite for this RasterSource.
 
         """
-        super(SRTM1Source, self).__init__(resolution=1, downloader=downloader,
-                                          max_nx=max_nx, max_ny=max_ny)
+        super().__init__(resolution=1, downloader=downloader,
+                         max_nx=max_nx, max_ny=max_ny)
 
 
 def srtm(lon, lat):
@@ -361,7 +345,7 @@ def read_SRTM(fh):
         elev.shape = (1201, 1201)
     else:
         raise ValueError(
-            'Shape of SRTM data ({}) is unexpected.'.format(elev.size))
+            f'Shape of SRTM data ({elev.size}) is unexpected.')
 
     fname = os.path.basename(fname)
     y_dir, y, x_dir, x = fname[0], int(fname[1:3]), fname[3], int(fname[4:7])
@@ -443,7 +427,7 @@ class SRTMDownloader(Downloader):
 
         if SRTMDownloader._SRTM_LOOKUP_MASK[lon, colat]:
             return (SRTMDownloader._SRTM_BASE_URL +
-                    u'{y}{x}.SRTMGL{resolution}.hgt.zip').format(**format_dict)
+                    '{y}{x}.SRTMGL{resolution}.hgt.zip').format(**format_dict)
         else:
             return None
 
@@ -457,9 +441,9 @@ class SRTMDownloader(Downloader):
         url = self.url(format_dict)
 
         srtm_online = self._urlopen(url)
-        zfh = ZipFile(six.BytesIO(srtm_online.read()), 'r')
+        zfh = ZipFile(io.BytesIO(srtm_online.read()), 'r')
 
-        zip_member_path = u'{y}{x}.hgt'.format(**format_dict)
+        zip_member_path = '{y}{x}.hgt'.format(**format_dict)
         member = zfh.getinfo(zip_member_path)
         with open(target_path, 'wb') as fh:
             fh.write(zfh.open(member).read())
@@ -490,7 +474,7 @@ class SRTMDownloader(Downloader):
         # dependencies of cartopy.
         from bs4 import BeautifulSoup
         if filename is None:
-            from six.moves.urllib.request import urlopen
+            from urllib.request import urlopen
             url = SRTMDownloader._SRTM_BASE_URL.format(resolution=resolution)
             with urlopen(url) as f:
                 html = f.read()
@@ -498,7 +482,7 @@ class SRTMDownloader(Downloader):
             with open(filename) as f:
                 html = f.read()
 
-        mask = np.zeros((360, 181), dtype=np.bool)
+        mask = np.zeros((360, 181), dtype=bool)
 
         soup = BeautifulSoup(html)
 
