@@ -1,24 +1,12 @@
-# (C) British Crown Copyright 2014 - 2020, Met Office
+# Copyright Cartopy Contributors
 #
-# This file is part of cartopy.
-#
-# cartopy is free software: you can redistribute it and/or modify it under
-# the terms of the GNU Lesser General Public License as published by the
-# Free Software Foundation, either version 3 of the License, or
-# (at your option) any later version.
-#
-# cartopy is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU Lesser General Public License for more details.
-#
-# You should have received a copy of the GNU Lesser General Public License
-# along with cartopy.  If not, see <https://www.gnu.org/licenses/>.
+# This file is part of Cartopy and is released under the LGPL license.
+# See COPYING and COPYING.LESSER in the root of the repository for full
+# licensing details.
 """This module contains tools for handling tick marks in cartopy."""
 
-from __future__ import (absolute_import, division, print_function)
-
 import numpy as np
+import matplotlib as mpl
 from matplotlib.ticker import Formatter, MaxNLocator
 
 import cartopy.crs as ccrs
@@ -34,16 +22,17 @@ class _PlateCarreeFormatter(Formatter):
 
     _target_projection = ccrs.PlateCarree()
 
-    def __init__(self, degree_symbol=u'\u00B0', number_format='g',
-                 transform_precision=1e-8, dms=False,
-                 minute_symbol=u"'", second_symbol=u"''",
+    def __init__(self, direction_label=True, degree_symbol='\u00B0',
+                 number_format='g', transform_precision=1e-8, dms=False,
+                 minute_symbol="'", second_symbol="''",
                  seconds_number_format='g',
-                 auto_hide=True):
+                 auto_hide=True, decimal_point=None, cardinal_labels=None):
         """
         Base class for simpler implementation of specialised formatters
         for latitude and longitude axes.
 
         """
+        self._direction_labels = direction_label
         self._degree_symbol = degree_symbol
         self._degrees_number_format = number_format
         self._transform_precision = transform_precision
@@ -55,6 +44,14 @@ class _PlateCarreeFormatter(Formatter):
         self._auto_hide_degrees = False
         self._auto_hide_minutes = False
         self._precision = 5  # locator precision
+        if (decimal_point is None and
+                mpl.rcParams['axes.formatter.use_locale']):
+            import locale
+            decimal_point = locale.localeconv()["decimal_point"]
+        if cardinal_labels is None:
+            cardinal_labels = {}
+        self._cardinal_labels = cardinal_labels
+        self._decimal_point = decimal_point
 
     def __call__(self, value, pos=None):
         if self.axis is not None and isinstance(self.axis.axes, GeoAxes):
@@ -87,16 +84,25 @@ class _PlateCarreeFormatter(Formatter):
         return self._format_value(projected_value, value)
 
     def _format_value(self, value, original_value):
-        hemisphere = self._hemisphere(value, original_value)
+
+        hemisphere = ''
+        sign = ''
+
+        if self._direction_labels:
+            hemisphere = self._hemisphere(value, original_value)
+        else:
+            if (value != 0 and
+                    self._hemisphere(value, original_value) in ['W', 'S']):
+                sign = '-'
 
         if not self._dms:
-            return (self._format_degrees(abs(value)) +
+            return (sign + self._format_degrees(abs(value)) +
                     hemisphere)
 
         value, deg, mn, sec = self._get_dms(abs(value))
 
         # Format
-        label = u''
+        label = ''
         if sec:
             label = self._format_seconds(sec)
 
@@ -104,7 +110,7 @@ class _PlateCarreeFormatter(Formatter):
             label = self._format_minutes(mn) + label
 
         if not self._auto_hide_degrees or not label:
-            label = self._format_degrees(deg) + hemisphere + label
+            label = sign + self._format_degrees(deg) + label + hemisphere
 
         return label
 
@@ -128,7 +134,7 @@ class _PlateCarreeFormatter(Formatter):
         degs = np.round(x, self._precision).astype('i')
         y = (x - degs) * 60
         mins = np.round(y, self._precision).astype('i')
-        secs = np.round((y - mins)*60, self._precision - 3)
+        secs = np.round((y - mins) * 60, self._precision - 3)
         return x, degs, mins, secs
 
     def set_locs(self, locs):
@@ -136,7 +142,7 @@ class _PlateCarreeFormatter(Formatter):
         if not self._auto_hide:
             return
         self.locs, degs, mins, secs = self._get_dms(self.locs)
-        secs = np.round(secs, self._precision-3).astype('i')
+        secs = np.round(secs, self._precision - 3).astype('i')
         secs0 = secs == 0
         mins0 = mins == 0
 
@@ -161,23 +167,18 @@ class _PlateCarreeFormatter(Formatter):
             number_format = 'd'
         else:
             number_format = self._degrees_number_format
-        return u'{value:{number_format}}{symbol}'.format(
-            value=abs(deg),
-            number_format=number_format,
-            symbol=self._degree_symbol)
+        value = f"{abs(deg):{number_format}}{self._degree_symbol}"
+        if self._decimal_point is not None:
+            value = value.replace(".", self._decimal_point)
+        return value
 
     def _format_minutes(self, mn):
         """Format minutes as an integer"""
-        return u'{value:d}{symbol}'.format(
-            value=int(mn),
-            symbol=self._minute_symbol)
+        return f'{int(mn):d}{self._minute_symbol}'
 
     def _format_seconds(self, sec):
         """Format seconds as an float"""
-        return u'{value:{fmt}}{symbol}'.format(
-            value=sec,
-            fmt=self._seconds_num_format,
-            symbol=self._second_symbol)
+        return f'{sec:{self._seconds_num_format}}{self._second_symbol}'
 
     def _apply_transform(self, value, target_proj, source_crs):
         """
@@ -202,10 +203,13 @@ class _PlateCarreeFormatter(Formatter):
 
 class LatitudeFormatter(_PlateCarreeFormatter):
     """Tick formatter for latitude axes."""
-    def __init__(self, degree_symbol=u'\u00B0', number_format='g',
+
+    def __init__(self, direction_label=True,
+                 degree_symbol='\u00B0', number_format='g',
                  transform_precision=1e-8, dms=False,
-                 minute_symbol=u"'", second_symbol=u"''",
+                 minute_symbol="'", second_symbol="''",
                  seconds_number_format='g', auto_hide=True,
+                 decimal_point=None, cardinal_labels=None
                  ):
         """
         Tick formatter for latitudes.
@@ -216,6 +220,11 @@ class LatitudeFormatter(_PlateCarreeFormatter):
 
         Parameters
         ----------
+        direction_label: optional
+            If *True* a direction label (N or S) will be drawn next to
+            latitude labels. If *False* then these
+            labels will not be drawn. Defaults to *True* (draw direction
+            labels).
         degree_symbol: optional
             The character(s) used to represent the degree symbol in the
             tick labels. Defaults to u'\u00B0' which is the unicode
@@ -241,6 +250,13 @@ class LatitudeFormatter(_PlateCarreeFormatter):
             values. Defaults to 'g'.
         auto_hide: bool, optional
             Auto-hide degrees or minutes when redundant.
+        decimal_point: bool, optional
+            Decimal point character. If not provided and
+            ``mpl.rcParams['axes.formatter.use_locale'] == True``,
+            the locale decimal point is used.
+        cardinal_labels: dict, optional
+            A dictionary with "south" and/or "north" keys to replace south and
+            north cardinal labels, which defaults to "S" and "N".
 
         Note
         ----
@@ -276,7 +292,8 @@ class LatitudeFormatter(_PlateCarreeFormatter):
             labels = [lat_formatter(value) for value in ticks]
 
         """
-        super(LatitudeFormatter, self).__init__(
+        super().__init__(
+            direction_label=direction_label,
             degree_symbol=degree_symbol,
             number_format=number_format,
             transform_precision=transform_precision,
@@ -285,16 +302,19 @@ class LatitudeFormatter(_PlateCarreeFormatter):
             second_symbol=second_symbol,
             seconds_number_format=seconds_number_format,
             auto_hide=auto_hide,
+            decimal_point=decimal_point,
+            cardinal_labels=cardinal_labels
         )
 
     def _apply_transform(self, value, target_proj, source_crs):
         return target_proj.transform_point(0, value, source_crs)[1]
 
     def _hemisphere(self, value, value_source_crs):
+
         if value > 0:
-            hemisphere = 'N'
+            hemisphere = self._cardinal_labels.get('north', 'N')
         elif value < 0:
-            hemisphere = 'S'
+            hemisphere = self._cardinal_labels.get('south', 'S')
         else:
             hemisphere = ''
         return hemisphere
@@ -304,16 +324,19 @@ class LongitudeFormatter(_PlateCarreeFormatter):
     """Tick formatter for a longitude axis."""
 
     def __init__(self,
+                 direction_label=True,
                  zero_direction_label=False,
                  dateline_direction_label=False,
-                 degree_symbol=u'\u00B0',
+                 degree_symbol='\u00B0',
                  number_format='g',
                  transform_precision=1e-8,
                  dms=False,
-                 minute_symbol=u"'",
-                 second_symbol=u"''",
+                 minute_symbol="'",
+                 second_symbol="''",
                  seconds_number_format='g',
                  auto_hide=True,
+                 decimal_point=None,
+                 cardinal_labels=None
                  ):
         """
         Create a formatter for longitudes.
@@ -323,6 +346,11 @@ class LongitudeFormatter(_PlateCarreeFormatter):
 
         Parameters
         ----------
+        direction_label: optional
+            If *True* a direction label (E or W) will be drawn next to
+            longitude labels. If *False* then these
+            labels will not be drawn. Defaults to *True* (draw direction
+            labels).
         zero_direction_label: optional
             If *True* a direction label (E or W) will be drawn next to
             longitude labels with the value 0. If *False* then these
@@ -356,6 +384,13 @@ class LongitudeFormatter(_PlateCarreeFormatter):
             values. Defaults to 'g'.
         auto_hide: bool, optional
             Auto-hide degrees or minutes when redundant.
+        decimal_point: bool, optional
+            Decimal point character. If not provided and
+            ``mpl.rcParams['axes.formatter.use_locale'] == True``,
+            the locale decimal point is used.
+        cardinal_labels: dict, optional
+            A dictionary with "west" and/or "east" keys to replace west and
+            east cardinal labels, which defaults to "W" and "E".
 
         Note
         ----
@@ -392,7 +427,8 @@ class LongitudeFormatter(_PlateCarreeFormatter):
             lon_formatter.set_locs(ticks)
             labels = [lon_formatter(value) for value in ticks]
         """
-        super(LongitudeFormatter, self).__init__(
+        super().__init__(
+            direction_label=direction_label,
             degree_symbol=degree_symbol,
             number_format=number_format,
             transform_precision=transform_precision,
@@ -401,6 +437,8 @@ class LongitudeFormatter(_PlateCarreeFormatter):
             second_symbol=second_symbol,
             seconds_number_format=seconds_number_format,
             auto_hide=auto_hide,
+            decimal_point=decimal_point,
+            cardinal_labels=cardinal_labels
         )
         self._zero_direction_labels = zero_direction_label
         self._dateline_direction_labels = dateline_direction_label
@@ -435,21 +473,22 @@ class LongitudeFormatter(_PlateCarreeFormatter):
         return _PlateCarreeFormatter._format_degrees(self, self._fix_lons(deg))
 
     def _hemisphere(self, value, value_source_crs):
+
         value = self._fix_lons(value)
         # Perform basic hemisphere detection.
         if value < 0:
-            hemisphere = 'W'
+            hemisphere = self._cardinal_labels.get('west', 'W')
         elif value > 0:
-            hemisphere = 'E'
+            hemisphere = self._cardinal_labels.get('east', 'E')
         else:
             hemisphere = ''
         # Correct for user preferences:
         if value == 0 and self._zero_direction_labels:
             # Use the original tick value to determine the hemisphere.
             if value_source_crs < 0:
-                hemisphere = 'E'
+                hemisphere = self._cardinal_labels.get('east', 'E')
             else:
-                hemisphere = 'W'
+                hemisphere = self._cardinal_labels.get('west', 'W')
         if value in (-180, 180) and not self._dateline_direction_labels:
             hemisphere = ''
         return hemisphere
@@ -465,8 +504,8 @@ class LongitudeLocator(MaxNLocator):
         Allow the locator to stop on minutes and seconds (False by default)
     """
 
-    default_params = MaxNLocator.default_params.copy()
-    default_params.update(nbins=8, dms=False)
+    def __init__(self, nbins=8, *, dms=False, **kwargs):
+        super().__init__(nbins=nbins, dms=dms, **kwargs)
 
     def set_params(self, **kwargs):
         """Set parameters within this locator."""
@@ -489,7 +528,7 @@ class LongitudeLocator(MaxNLocator):
             steps = np.array([1, 1.5, 2, 2.5, 3, 5, 10])
 
         else:
-            steps = np.array([1, 10/6., 15/6., 20/6., 30/6., 10])
+            steps = np.array([1, 10 / 6., 15 / 6., 20 / 6., 30 / 6., 10])
 
         self.set_params(steps=np.array(steps))
 
@@ -511,6 +550,7 @@ class LatitudeLocator(LongitudeLocator):
     dms: bool
         Allow the locator to stop on minutes and seconds (False by default)
     """
+
     def tick_values(self, vmin, vmax):
         vmin = max(vmin, -90.)
         vmax = min(vmax, 90.)
